import 'package:example/dto.dart';
import 'package:flutter/foundation.dart';
import 'package:flutterx_http/flutterx_http.dart';

class OpenWeather {
  static final ApiCall<WeatherInfo> _getWeather = ApiCall.object<WeatherInfo>(
      request: (client, params) => client.get('/weather', params: params), fromJson: WeatherInfo.fromJson);
  final XHttpClient _client;

  OpenWeather(String apiKey)
      : _client = XHttpClient(
            baseUrl: 'https://api.openweathermap.org/data/2.5',
            onRequest: (request) => request.copy(url: request.url.appendQueryParameters({'appid': apiKey})));

  Future<WeatherInfo> getWeather({required String city, Units units = Units.standard}) =>
      _getWeather(_client, {'q': city, 'units': describeEnum(units)});

  String imageUrl(WWeather weather) => 'http://openweathermap.org/img/wn/${weather.icon}@2x.png';
}

enum Units { standard, metric, imperial }

extension UnitsExt on Units {
  String get unit {
    switch (this) {
      case Units.standard:
        return '°K';
      case Units.metric:
        return '°C';
      case Units.imperial:
        return '°F';
    }
  }
}
